# Copyright (c) 1993 Xerox Corporation.
# Use and copying of this software and preparation of derivative works based
# upon this software are permitted. Any distribution of this software or
# derivative works must comply with all applicable United States export
# control laws. This software is made available AS IS, and Xerox Corporation
# makes no warranty about the software, its performance or its conformity to
# any specification.

# changes/additions by Jason Rennie <jr6b+@andrew.cmu.edu>

###############
### inc.tcl ###
###############

proc Inc_Init {} {
    global exmh inc
    if {$exmh(slocal) == {}} {
	set exmh(slocal) slocal
    }
    if [info exists exmh(incStyle)] {
	# Backward compatibility
	set inc(style) $exmh(incStyle)
    }
    if [info exists exmh(incOnStart)] {
	# Backward compatibility
	set inc(onStartup) $exmh(incOnStart)
    }
    Preferences_Add "Incorporate Mail" \
"Exmh knows several ways to incorporate mail from your system's spool file into the MH folder hierarchy." {
	{inc(style) incStyle	{CHOICE inbox presort multidrop presortmulti ifile none} {Ways to Inc}
"inbox - basic MH inc from your spool file to inbox folder.
presort - slocal filtering directly into various folders.
multidrop - slocal filtering or POP delivery into various drop boxes,
as specified by ~/.xmhcheck, that are in turn inc'ed into folders.
presortmulti - presort + multidrop.
ifile - intelligent message content filtering directly into various folders.
Please read ifile.README for more detailed information.
none - you have an external agent that handles inc for you, so
don't bother trying it from within exmh."}
	{inc(onStartup) incOnStartup OFF	  {Inc at startup}
"Run your flavor of inc when exmh starts up."}
	{inc(onMapping) incOnMapping OFF	  {Inc on mapping}
"Run your flavor of inc when exmh is opened
after being iconic."}
	{inc(presortFeedback) incSortFeedback ON	  {Presort Inc feedback}
"This option causes a message to be generated each time a
new message is sorted into a folder.  There isn't much info,
it's just sort of a heart-beat feature..."}
	{inc(xnsgetmail) xnsGetMail OFF	  {Run xnsgetmail -k}
"Run xnsgetmail -k before inc in order to fetch your
XNS mail messages into your UNIX spool file."}
    }
}

proc IncInner {} {
    # There are four different styles of inc'ing messages...
    global inc
    if $inc(xnsgetmail) {
	Xns_GetMail
    }
    case $inc(style) {
	{default inbox}	{ busy Inc_Inbox }
	presort		{ busy Inc_Presort }
	multidrop	{ busy Inc_All }
	presortmulti	{ busy Inc_PresortMultidrop }
	ifile		{ busy Inc_iFile }
	none		{ return }
    }
}

proc Inc_iFile {} {
    # Transfer from the mail spool file directly into folders
    global exmh mhProfile env inc
    # Use inc to copy stuff into a temp directory
    if ![file isdirectory $mhProfile(path)/MyIncTmp] {
      exec mkdir $mhProfile(path)/MyIncTmp
    }
    if {[catch {exec inc +MyIncTmp -silent} err]} {
      # Some versions of inc exit non-zero when they should not.
      Exmh_Debug Inc_Presort +MyIncTmp: $err
    }
    if [catch {set env(USER)} user] {
      if [catch {set env(LOGNAME)} user] {
          Exmh_Status "No USER or LOGNAME envar"
          set user ""
      }
    }
    if {[catch {glob $mhProfile(path)/MyIncTmp/*} files] == 0} {
      Exmh_Status "ifile inc ..."
      foreach file $files {
          if {[string compare [file tail $file] "++"] == 0} {
              if [catch {exec rm $file} err] {
                  Exmh_Debug Inc_iFile: $err
              }
              continue
          }
          if {$inc(presortFeedback)} {
              if [catch {exec grep "^Subject:" $file | head -1} \
                  subject] {
                  set subject ""
              }
          }

          # now use ifilter to (presumably) shift things to a folder
          if [catch {exec ifilter.mh < $file} err] {
              # Move it out of MyIncTmp in case it really did
              # get filed somewhere.  Certain file system errors
              # (can't stat .) lead to this behavior
              Exmh_Status "$file - $err - check MyIncErrors"
              if ![file isdirectory $mhProfile(path)/MyIncErrors] {
                  exec mkdir $mhProfile(path)/MyIncErrors
              }
              Mh_Refile MyIncTmp [file tail $file] MyIncErrors 
          } else {
              if [catch {exec rm $file} err] {
                  Exmh_Debug Inc_iFile: $err
              }
              if {$inc(presortFeedback)} {
                  if {[string length $subject] > 0} {
                      regsub "^Subject: *" $subject "" subject
                      Exmh_Status "[file tail $file]: $subject" red
                  } else {
                      Exmh_Status [file tail $file] red
                  }
              }
          }
      }
    }
    catch {exec rm -f $mhProfile(path)/MyIncTmp/$mhProfile(mh-sequences)}
    catch {exec rmdir $mhProfile(path)/MyIncTmp}

    Flist_FindUnseen          ;# Needed to set new message state.
    # This after breaks a potential deadlock:
    # UI Inc button is pressed - registers outstanding Inc operation
    # PresortFinish notes new messages in the current folder and wants
    # to rescan the folder to pick up the changes.
    # The scan notes the outstanding operation and waits for it to complete.
    # Deadlock
    after 1 {BgRPC Inc_iFileFinish}
}
proc Inc_iFileFinish {} {
    global exmh ftoc
    Mh_Folder $exmh(folder)   ;# Inc_iFile inc has changed this to MyIncTmp
    if {$ftoc(displayValid) && [Flist_NumUnseen $exmh(folder)] > 0} {
      Label_Folder $exmh(folder)
      Scan_Folder $exmh(folder) $ftoc(showNew)
      }
  }

###############
### msg.tcl ###
###############

# this procedure filters a message using ifilter
proc Msg_Filter {} {
    global ftoc exmh mhProfile

    set msg_num [Ftoc_MsgNumber $ftoc(curLine)]

    Exmh_Status "ifile filter ..."

    if [catch {exec ifilter.mh < "$mhProfile(path)/$exmh(folder)/$msg_num"} err] {
	Exmh_Debug Inc_iFile: $err
    } else {
	Msg_Remove
	Flist_FindUnseen 1
    }
}

##########
# mh.tcl #
##########

proc Mh_Refile {srcFolder msgs folder} {
    while {[llength $msgs] > 0} {
	set chunk [lrange $msgs 0 19]
	set msgs [lrange $msgs 20 end]
	eval {MhExec irefile.mh} $chunk {-src +$srcFolder +$folder}
    }
}
